/**
 * \file    main.c
 * W5300 example applications
 * 
 * This file shows how to use  SOCKET_API_FUNC "WIZnet SOCKET APIs" and 
 * how to implement your internet application with  W5300.\n
 * 
 * \author MidnightCow
 * \date 04/07/2008
 * \version 1.1.1
 *
 * Revision History :
 * ----------  -------  -----------  ----------------------------
 * Date        Version  Author       Description
 * ----------  -------  -----------  ----------------------------
 * 24/03/2008  1.0.0    MidnightCow  Release with W5300 launching
 * ----------  -------  -----------  ----------------------------
 * 01/05/2008  1.0.1    MidnightCow  Modify 'w5300.c'. Refer M_01052008.
 * ----------  -------  -----------  ----------------------------
 * 15/05/2008  1.1.0    MidnightCow  Refer M_15052008.
 *                                   Modify 'w5300.c', 'w5300.h' and 'socket.c'. 
 * ----------  -------  -----------  ----------------------------
 * 04/07/2008  1.1.1    MidnightCow  Refer M_04072008.
 *                                   Modify 'socket.c'. 
 * ----------  -------  -----------  ----------------------------
 * 08/08/2008  1.2.0    MidnightCow  Refer M_08082008.
 *                                   Modify 'w5300.c' & 'socket.c'. 
 * ----------  -------  -----------  ----------------------------
 *
 *
 * The Main() includes the example program of  W5300 such as 
 * - loopback_tcps() \n
 * - loopback_tcpc() \n
 * - loopback_udp()
 */



#include "main_w.h"
#include "xscale.h"
#include "socket.h"

uint8 g_data_buf[0x4000];

LOADER_STATUS	status;           // for serial port

void InitXHyper255A(void);       // Intialize MCU

/* Definition of example code */
void     loopback_tcps(SOCKET s, uint16 port, uint8* buf,uint16 mode);
void     loopback_tcpc(SOCKET s, uint8* addr, uint16 port, uint8* buf,uint16 mode);
void     loopback_udp(SOCKET s, uint16 port, uint8* buf, uint16 mode);
 
void get_mac_address(unsigned char *mac);

/**
 * It executes example program such as loopback_tcps(), loopback_tcpc(), and loopback_udp().
 */ 
void w5300_main()
{
   static uint8 tx_mem_conf[8] = {8,8,8,8,8,8,8,8};          // for setting TMSR regsiter
   static uint8 rx_mem_conf[8] = {8,8,8,8,8,8,8,8};          // for setting RMSR regsiter
   
//   uint8 * g_data_buf = (uint8 *)0x0C000000; //Top of SDRAM
   
//   uint8 * g_data_buf = (uint8 *) 0xA10E0000;         // buffer for loopack data
   
   static uint8 ip[4] = {192,168,1,111};                   // for setting SIP register
   static uint8 gw[4] = {192,168,1,1};                     // for setting GAR register
   static uint8 sn[4] = {255,255,255,0};                     // for setting SUBR register
//   uint8 mac[6] = {0x00,0x18,0x5f,0x01,0x02,0x03};      // for setting SHAR register
   static uint8 mac[6];      // for setting SHAR register
   
//   static uint8 serverip[4] = {192,168,24,75};              // "TCP SERVER" IP address for loopback_tcpc()
//   static uint8 serverip[4] = {192,168,1,3};              // "TCP SERVER" IP address for loopback_tcpc()
   
   
   
//   status.terminalSpeed = SERIAL_SPEED;
//   status.downloadSpeed = SERIAL_DOWNLOAD_SPEED;
   
//   g_data_buf = (uint8*)0xA10F0000;                       
   

   get_mac_address(mac);
   
   
   /* initiate W5300 */
   iinchip_init();  



   /* allocate internal TX/RX Memory of W5300 */
   if(!sysinit(tx_mem_conf,rx_mem_conf))           
   {
      w_printf("MEMORY CONFIG ERR.\r\n");
      while(1);
   }


   //setMR(getMR()|MR_FS);                            // If Little-endian, set MR_FS.
   
   setSHAR(mac);                                      // set source hardware address
   
   #ifdef __DEF_IINCHIP_PPP__
      if(pppinit((uint8*)"test01", 6, (uint8*)"pppoe1000", 9)!=1)
      {
         w_printf("PPPoE fail.\r\n");
         while(1);
      }
      close(0);
   #else
      /* configure network information */
      setGAR(gw);                                     // set gateway IP address
      setSUBR(sn);                                    // set subnet mask address
      setSIPR(ip);                                    // set source IP address
   #endif
   
   /* verify network information */
   getSHAR(mac);                                      // get source hardware address 
   getGAR(gw);                                        // get gateway IP address      
   getSUBR(sn);                                       // get subnet mask address     
   getSIPR(ip);                                       // get source IP address       
   
   w_printf("SHAR : %02x:%02x:%02x:%02x:%02x:%02x\r\n",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
   w_printf("GWR  : %d.%d.%d.%d\r\n",gw[0],gw[1],gw[2],gw[3]);
   w_printf("SUBR : %d.%d.%d.%d\r\n",sn[0],sn[1],sn[2],sn[3]);
   w_printf("SIPR : %d.%d.%d.%d\r\n",ip[0],ip[1],ip[2],ip[3]);
   
   
   while(1)
   {
      
      loopback_tcps(0,1000,g_data_buf,0);
      loopback_tcps(1,2000,g_data_buf,0);
      loopback_tcps(2,3000,g_data_buf,0);
      loopback_tcps(3,4000,g_data_buf,0);
      loopback_tcps(4,5000,g_data_buf,0);
      
      loopback_tcps(5,6000,g_data_buf,0);
      //loopback_tcpc(6,serverip, 7000, g_data_buf,0); //TCPNCAg
      loopback_udp(7,8000,g_data_buf,0);
   }

   #ifdef __DEF_IINCHIP_PPP__
   {
      uint8 ppp_mac[6];
      getPDHAR(ppp_mac);
      pppterm(ppp_mac, getPSIDR());
   }
   #endif

   while(1);

}


void InitXHyper255A(void)
{
   ADDR32(GPIO_BASE+GPDR2) |= GPDR2_VALUE; //~0x0;//((ADDR32(GPIO_BASE+GPDR2) & 0x0001FFFF) | GPDR2_VALUE);
   ADDR32(GPIO_BASE+GAFR2_L) |= GAFR2L_VALUE;
   ADDR32(MEM_CTL_BASE+MSC1) = 0x00008259;//0x000095F8;
}

/**
 * "TCP SERVER" loopback program.
 */ 
void     loopback_tcps(SOCKET s, uint16 port, uint8* buf, uint16 mode)
{
   uint32 len;
   
   uint8 destip[4] = {192,168,24,251}; //for dummy
   uint16 destport = 8000;             //for dummy
   
   
   switch(getSn_SSR(s))                // check SOCKET status
   {                                   // ------------
      case SOCK_ESTABLISHED:           // ESTABLISHED?
         if(getSn_IR(s) & Sn_IR_CON)   // check Sn_IR_CON bit
         {
            w_printf("%d : Connect OK\r\n",s);
            setSn_IR(s,Sn_IR_CON);     // clear Sn_IR_CON
         }
         if((len=getSn_RX_RSR(s)) > 0) // check the size of received data
         {
            len = recv(s,buf,len);     // recv
			w_printf("test:%s \r\n",buf);
            if(len !=send(s,buf,len))  // send
            {
               w_printf("%d : Send Fail.len=%d\r\n",s,len);
            }
         }
         break;

                                       // ---------------
   case SOCK_CLOSE_WAIT:               // PASSIVE CLOSED
         disconnect(s);                // disconnect 
         w_printf("%d : disconnect \r\n",s);
//         break;
                                       // --------------
   case SOCK_CLOSED:                   // CLOSED
      //dummy
      //socket(s,Sn_MR_UDP,destport,mode);   // open the SOCKET  
      //sendto(s,buf,100,destip,destport);  // send the data to the destination
      //
      close(s);                        // close the SOCKET
      w_printf("%d : close \r\n",s);
      socket(s,Sn_MR_TCP,port,mode);   // open the SOCKET  
      break;
                                       // ------------------------------
   case SOCK_INIT:                     // The SOCKET opened with TCP mode
      listen(s);                       // listen to any connection request from "TCP CLIENT"
      w_printf("%d : LOOPBACK_TCPS(%d) Started.\r\n",s,port);
      break;
   default:
      break;
   }
}

/**
 * "TCP CLIENT" loopback program.
 */ 
void     loopback_tcpc(SOCKET s, uint8* addr, uint16 port, uint8* buf, uint16 mode)
{
   uint32 len;
   static uint16 any_port = 1000;
   
   switch(getSn_SSR(s))                   // check SOCKET status
   {                                      // ------------
      case SOCK_ESTABLISHED:              // ESTABLISHED?
         if(getSn_IR(s) & Sn_IR_CON)      // check Sn_IR_CON bit
         {
            w_printf("%d : Connect OK\r\n",s);
            setSn_IR(s,Sn_IR_CON);        // clear Sn_IR_CON
         }
         if((len=getSn_RX_RSR(s)) > 0)    // check the size of received data
         {
            len = recv(s,buf,len);        // recv
			w_printf("test:%s \r\n",buf);
            if(len !=send(s,buf,len))     // send
            {
               w_printf("%d : Send Fail.len=%d\r\n",s,len);
            }
         }
         break;
                                          // ---------------
   case SOCK_CLOSE_WAIT:                  // PASSIVE CLOSED
         disconnect(s);                   // disconnect 
         break;
                                          // --------------
   case SOCK_CLOSED:                      // CLOSED
      close(s);                           // close the SOCKET
      socket(s,Sn_MR_TCP,any_port++,mode);// open the SOCKET with TCP mode and any source port number
      break;
                                          // ------------------------------
   case SOCK_INIT:                        // The SOCKET opened with TCP mode
      connect(s, addr, port);             // Try to connect to "TCP SERVER"
      w_printf("%d : LOOPBACK_TCPC(%d.%d.%d.%d:%d) Started.\r\n",s,addr[0],addr[1],addr[2],addr[3],port);
      break;
   default:
      break;
   }
}

/**
 * UDP loopback program.
 */ 
void     loopback_udp(SOCKET s, uint16 port, uint8* buf, uint16 mode)
{
   uint32 len;
   uint8 destip[4];
   uint16 destport;
   
   switch(getSn_SSR(s))
   {
                                                         // -------------------------------
      case SOCK_UDP:                                     // 
         if((len=getSn_RX_RSR(s)) > 0)                   // check the size of received data
         {
            w_printf("len = %d \r\n",len);
            len = recvfrom(s,buf,len,destip,&destport);  // receive data from a destination
            if(len !=sendto(s,buf,len,destip,destport))  // send the data to the destination
            {
               w_printf("%d : Sendto Fail.len=%d,",s,len);
               w_printf("%d.%d.%d.%d(%d)\r\n",destip[0],destip[1],destip[2],destip[3],destport);
            }
         }
         break;
                                                         // -----------------
      case SOCK_CLOSED:                                  // CLOSED
         close(s);                                       // close the SOCKET
         socket(s,Sn_MR_UDP,port,mode);                  // open the SOCKET with UDP mode
         break;
      default:
         break;
   }
}
