/*----------------------------------------------*/
/* SPI0 GPIO  Control Module                          */
/*----------------------------------------------*/

#include "iodefine.h"
//#include "scif.h"
//#include "vect.h"


/* Control signals (Platform dependent) */
#define EEP_CS_HIGH()	(PORT.PKDR0.BIT.PK7DR = 1)		/* EEP CS = H */
#define EEP_CS_LOW()	(PORT.PKDR0.BIT.PK7DR = 0)		/* EEP CS = L */

#define EEP_DO_HIGH()	(PORT.PFDR0.BIT.PF11DR = 1)		/* EEP DO = H */
#define EEP_DO_LOW()	(PORT.PFDR0.BIT.PF11DR = 0)		/* EEP DI = L */

#define EEP_CLK_HIGH()	(PORT.PFDR0.BIT.PF9DR = 1)		/* EEP DO = H */
#define EEP_CLK_LOW()	(PORT.PFDR0.BIT.PF9DR = 0)		/* EEP DI = L */

#define EEP_DI (PORT.PFPR0.BIT.PF12PR) 

#define DIO_PA (*(volatile unsigned char *)(0x30000640))

void wait_eep(int a){
	volatile int i;
	
	for(i=0;i<(a * 3);i++){ ; }
}
void init_spi0_gpio(void)
{
	int i;
	int i_b = 0;

//	EEP_CS_HIGH();
	EEP_CS_LOW();
	EEP_DO_LOW();
	EEP_CLK_LOW();
	
//	PORT.PKDR0.BIT.PK7DR = 1; //PK7 CS H
//	PORT.PFDR0.BIT.PF11DR = 0; //MO PF11  L
//	PORT.PFDR0.BIT.PF9DR = 0; //CLK PF9  L

	PORT.PKIOR0.BIT.PK7IOR = 1; //DIR OUT CS
	PORT.PFIOR0.BIT.PF12IOR = 0; //DIR IN DI
	PORT.PFIOR0.BIT.PF11IOR = 1; //DIR OUT DO
	PORT.PFIOR0.BIT.PF9IOR = 1;  //DIR OUT CLK
	
	PORT.PKCR1.BIT.PK7MD = 0; //GPIO CS 93C46
	PORT.PFCR3.BIT.PF12MD = 0; //GPIO DI
	PORT.PFCR2.BIT.PF11MD = 0; //GPIO DO
	PORT.PFCR2.BIT.PF9MD = 0; //GPIO CLK


}

/* SPI out          */
void xmit_spi (unsigned short dat,int n)
{
	unsigned short mask;
	
	if(n == 9)
		mask = 0x100;
	else if(n == 11)
		mask = 0x400;
	else
		mask = 0x8000;
	
	
	do {
		if(dat & mask){
			EEP_DO_HIGH();
		}
		else{
			EEP_DO_LOW();
		}
		wait_eep(100);
		dat <<= 1;
		
		EEP_CLK_HIGH();
		wait_eep(100);
		EEP_CLK_LOW();
		wait_eep(100);
	
	} while (--n);
}
/* SPI in          */
unsigned short rcvr_spi()
{
	unsigned short r = 0, n = 16;


	EEP_DO_LOW();
	wait_eep(100);
	do {
		r <<= 1;
		EEP_CLK_HIGH();
		wait_eep(100);
		EEP_CLK_LOW();
		wait_eep(100);
		if (EEP_DI) r++;
	} while (--n);
	return r;
}

unsigned short read_eep(unsigned short add){
	unsigned short data = 0;
		
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	wait_eep(500);
	
	xmit_spi(0x180 | add,9); //93C46
//	xmit_spi(0x600 | add,11); //93C56
	data = rcvr_spi();
	
	EEP_CS_LOW();
	return data;
}
int write_eep(unsigned short add,unsigned short data){
		
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	wait_eep(500);
	
	xmit_spi(0x140 | add,9); //93C46
//	xmit_spi(0x500 | add,11); //93C56
	xmit_spi(data,16);
	
	EEP_CS_LOW();
	wait_eep(2000);
	
	if(read_eep(add) != data)
		return 1;
	else
		return 0;
	
}
unsigned short read_eep_s(unsigned short add){
	unsigned short data = 0;
		
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	wait_eep(500);
	
	xmit_spi(0x180 | add,9);
	data = rcvr_spi();
	
	EEP_CS_LOW();
	return data;
}
int write_eep_s(unsigned short add,unsigned short data){
		
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	EEP_CS_HIGH();
	wait_eep(500);
	
	xmit_spi(0x140 | add,9);
	
	xmit_spi(data,16);
	
	EEP_CS_LOW();
	wait_eep(500);
	
	if(read_eep_s(add) != data)
		return 1;
	else
		return 0;
	
}
unsigned short bswap(unsigned short a){
	return( (a >> 8 & 0x00ff) | (a << 8 & 0xff00) );
}
void test_eep(void){
	volatile unsigned short data_b[20];
	int i;
	
	for(i=0;i<20;i++)
	{
		data_b[i] = (read_eep_s(i));
	}
	
}


void get_mac_address(unsigned char *mac)
{
	volatile unsigned short data_b[40];
	int i; 
	int ee_cnt = 0;
	
	init_spi0_gpio();
	
	for(i=2;i<20;i++) //mac addres 2
	{
		//data_b[i] = bswap(read_eep_s(i));
		data_b[i] = (read_eep_s(i));
		mac[ee_cnt++] = data_b[i] & 0xff;
		mac[ee_cnt++] = ((data_b[i] >> 8) & 0xff);
	}
	
}






