/*----------------------------------------------*/
/* SPI1 Control Module                          */
/*----------------------------------------------*/


#include "iodefine.h"
#include <machine.h>
//#include "sh7262board_iic.h"
#include "W5300/lstring.h"
//#include "sh7262board_timer.h"


/* ATA command */
#define CMD_RESET		0x08	/* DEVICE RESET */
#define CMD_READ		0x20	/* READ SECTOR(S) */
#define CMD_WRITE		0x30	/* WRITE SECTOR(S) */
#define CMD_IDENTIFY	0xEC	/* DEVICE IDENTIFY */
#define CMD_SETFEATURES	0xEF	/* SET FEATURES */

/* ATA register bit definitions */
#define	LBA				0x0040
#define	BSY				0x0080
#define	DRDY			0x0040
#define	DF				0x0020
#define	DRQ				0x0008
#define	ERR				0x0001
#define	SRST			0x0040
#define	nIEN			0x0020

/* Bit definitions for Control Port */
#define	REG_DATA		(*(volatile unsigned short *)(0x28000000)) 
#define	REG_ERROR		(*(volatile unsigned short *)(0x28000002)) 
#define	REG_FEATURES	(*(volatile unsigned short *)(0x28000002)) 
#define	REG_COUNT		(*(volatile unsigned short *)(0x28000004)) 
#define	REG_SECTOR		(*(volatile unsigned short *)(0x28000006)) 
#define	REG_CYLL		(*(volatile unsigned short *)(0x28000008)) 
#define	REG_CYLH		(*(volatile unsigned short *)(0x2800000a)) 
#define	REG_DEV			(*(volatile unsigned short *)(0x2800000c)) 
#define	REG_COMMAND		(*(volatile unsigned short *)(0x2800000e)) 
#define	REG_STATUS		(*(volatile unsigned short *)(0x2800000e)) 
#define	REG_DEVCTRL		(*(volatile unsigned short *)(0x2800002c)) 
#define	REG_ALTSTAT		(*(volatile unsigned short *)(0x2800002c)) 



/* Control signals (Platform dependent) */
#define MAX1270_CS_HIGH()	(PORT.PGDR1.BIT.PG22DR = 1)		/* MAX1270 CS = H */
#define MAX1270_CS_LOW()	(PORT.PGDR1.BIT.PG22DR = 0)		/* MAX1270 CS = L */

#define BCLK		48000000UL			/* SPIW[NbN */
#define CLK_FAST	12000000UL			/* SCLK */
//#define CLK_FAST	8000000UL			/* SCLK */
#define	CLK_SLOW	2000000UL			/* SCLKᑬ () */
#define	DIV_F		(BCLK/CLK_FAST/2-1)
#define	DIV_S		(BCLK/CLK_SLOW/2-1)
#define	FCLK_SLOW1()	{RSPI1.SPCR.BIT.SPE = 0; RSPI1.SPBR.BYTE=DIV_S; RSPI1.SPCR.BIT.SPE = 1;}	/* Set slow clock (100k-400k) */
#define	FCLK_FAST1()	{RSPI1.SPCR.BIT.SPE = 0; RSPI1.SPBR.BYTE=DIV_F; RSPI1.SPCR.BIT.SPE = 1;}	/* Set fast clock (MMC:20MHz max, SD:25MHz max) */

volatile unsigned short ide_data[512];
volatile unsigned short ide_data_b[512 * 2];


void delay_ms(unsigned int);				

void init_ad_gpio(void)
{
	int i;
	int i_b = 0;

	MAX1270_CS_HIGH();
//	MAX1270_CS_LOW();
	
	PORT.PGIOR1.BIT.PG22IOR = 1;  //DIR OUT CS MAX1270
	PORT.PGCR5.BIT.PG22MD = 0; //GPIO PG22
}

int spi1_Putc(unsigned char c) {
	RSPI1.SPDR.BYTE = c;			/* MJn */
	while (!RSPI1.SPSR.BIT.SPRF) ;	/* ]I҂ */
	RSPI1.SPDR.BYTE;				/* Mf[^͊Ă */
	return 0;
}
unsigned char spi1_Gettc(void) {
	unsigned char data;
	RSPI1.SPDR.BYTE = 0x0;			/* MJn */
	while (!RSPI1.SPSR.BIT.SPRF) ;	/* ]I҂ */
	data = RSPI1.SPDR.BYTE;				/* Mf[^͊Ă */
	return data;
}
int get_ad_max1270(char ch,char rng){
	//unsigned char ch = 0;
	//unsigned char rng = 2; //0-10V
	unsigned char ad_data[4];
	unsigned short *ad_value;
	
	ad_value = (unsigned short *)ad_data;
	
	ch = 0x80 | ((ch << 4) & 0x70) | ((rng << 2) & 0xc) | 0x1;
		
	MAX1270_CS_LOW();
		
	spi1_Putc(ch);
	
	ad_data[0] = spi1_Gettc();
	ad_data[1] = spi1_Gettc();
	ad_data[2] = spi1_Gettc();
	
	MAX1270_CS_HIGH();
	
	ad_value[0] <<= 1;
	if(ad_data[2] & 0x80) ad_value[0]++;
	
	return (int)ad_value[0];
}


unsigned short Timer_e;		/* 10msec */
void init_ide(void){
	
	REG_DEV = 0; 					/* Select Device 0 */
	for (Timer_e = 1; Timer_e; );				/* 10ms */
	REG_DEV = 0; 					/* Select Device 0 */

	do {
	} while (!((REG_STATUS) & (BSY | DRQ)));

	REG_DEVCTRL =  (SRST | nIEN);	/* Software reset */
	for (Timer_e = 2; Timer_e; );
	REG_DEVCTRL =  nIEN;
	for (Timer_e = 2; Timer_e; );

	do {
	} while (((REG_STATUS) & (DRDY|BSY)) != DRDY);

	REG_FEATURES =  3;				/* Select PIO default mode without IORDY */
	REG_COUNT = 1;
	REG_COMMAND =  CMD_SETFEATURES;
	do {
	} while ((REG_STATUS) & BSY);
}

void wait_data(void){
	unsigned short s;
	int dummy;
	
	do {
		s = (REG_STATUS);		/* Get status */
	} while ((s & (BSY|DRQ)) != DRQ);	/* Wait for BSY goes low and DRQ goes high */

	dummy = (REG_ALTSTAT);
}

void ide_test(void){
  int i,i_b;
   
   //init_ide();
   
	PORT.PJIOR0.BIT.PJ11IOR = 1;  //DIR OUT CS MAX1270
	PORT.PJCR2.BIT.PJ11MD = 0; //GPIO PG22
    PORT.PJDR0.BIT.PJ11DR = 1;
   
//   for(;;){
//   IDE_COMMAND = 0xa1;
   REG_COMMAND = 0xec;
   
   for (Timer_e = 4; Timer_e; );
   
   i_b = 0;
   for(i=0;i<512;i++){
     ide_data[i] = REG_DATA; 
   	 //ide_data_b[i_b++] = ide_data[i] & 0xff;
  	 //ide_data_b[i_b++] = (ide_data[i] >> 8) & 0xff;
   }
//   }
   
   for(i=0;i<(512);i++){ 
 		w_printf("%03d %04x \r\n",i, ide_data[i]); 
// 		if((i % 8) == 0){
//			w_printf("\r\n");
//		}
   }
   
   for(;;) i_b = 0;
}
void init_SPI1(void){
	CPG.STBCR5.BIT.MSTP50 = 0;	/* SPI1W[L */

   	/* SPI1W[I/Osɐڑ */
	PORT.PGCR5.BIT.PG21MD = 1; //RSPCK1
	PORT.PGCR5.BIT.PG23MD = 1; //MOSI1
	PORT.PGCR6.BIT.PG24MD = 1; //MISO1
	
	PORT.PGIOR1.BIT.PG21IOR = 1; //DIR OUT
	PORT.PGIOR1.BIT.PG23IOR = 1; //DIR OUT
	PORT.PGIOR1.BIT.PG24IOR = 0; //DIR IN

	/* SPI0W[ */
	RSPI1.SPPCR.BYTE = 0;			/* AChlŒƃ[vobN[h: gp */
	RSPI1.SPSCR.BYTE = 0;			/* V[PX: gp */
	RSPI1.SPDCR.BYTE = 0x20;		/* FIFOANZX: oCg */
	RSPI1.SPBFCR.BYTE = 0xF0; RSPI1.SPBFCR.BYTE = 0x30;	/* FIFO */
	RSPI1.SPCMD0.WORD = 0x0700;		/* SPI[h: MODE0, 8bit, MSB first */
	RSPI1.SPCR.BYTE = 0x48;			/* SPICl[u(}X^[h) */

    FCLK_SLOW1();
}
void test_max1270(void){
	int i;
	unsigned char ad_data[10];
	
	
	init_ad_gpio();
	
	init_SPI1();
	
	for(;;){
		for(i=0;i<8;i++){
			w_printf("%04x ",get_ad_max1270(i,2)); //0-10V
			//w_printf("%04x ",get_ad_max1270(i,0)); //0-5V
			//w_printf("%04x ",get_ad_max1270(i,3)); //-10V - +10V
		}
		w_printf("\r\n");
	 	delay_ms(50);
	}
	
}









